// STB_volavg 0.0.2 2010:08:11 00:00:00$
// Copyright (c) 2010 solgracia
// Released under the GNU General Public License
// 
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
    /// The STB_volavg plots volume against volume averages.
    /// Reproduces the logic described by Lorn at http://www.mypivots.com/
    /// http://www.mypivots.com/board/topic/5997/8/es-short-term-trading-8-5-2010
	/// </summary>
    [Description("The STB_volavg plots volume against volume averages.")]
	public class STB_volavg : Indicator
	{
		#region Variables
        private int alpha;

        private bool showmsg = false;          // show msg
        private int shift = 0;          // shift bars ahead
        private int depth = 5;          // number of days back for avg
        private int distance = 1;       // days back to start calculating avg
        private int jump = 1;          // jump days for avg sample
        private int transparency = 60;          // transparency of avg color

        private Pen volumeAvgPen = null;
        private Pen volumeDataPen = null;
        private Pen volumePerPen = null;

        private Color volumeDataColor = Color.Gold;
        private Color volumeAvgColor = Color.DodgerBlue;

        private DataSeries va;
        private DataSeries vd;

        #endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{

            int alpha	= (int) (255.0 * ((100.0 - transparency) / 100.0));
            Pen volumeDataPen = new Pen(Color.FromArgb(alpha, volumeDataColor.R, volumeDataColor.G, volumeDataColor.B));
            Pen volumeAvgPen = new Pen(volumeAvgColor);

            Add(new Plot(volumeAvgPen, "VOL_AVG"));                     // 0 - line
            Add(new Plot(volumeDataPen, PlotStyle.Bar, "VOL_DATA"));    // 1 - bars
            Add(new Line(Color.DarkGray, 0, "Zero line"));

            CalculateOnBarClose = true; 
            Overlay = false;

            RecalculateColors();

            Displacement = shift;

            va = new DataSeries(this);
            vd = new DataSeries(this);

		}

		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
            if (Bars == null) return;  

            DateTime startTime = Bars.Get(0).Time;
            DateTime suppTime = startTime.AddDays(1 + (depth + distance) * jump);
            DateTime barTime = Bars.Get(CurrentBar).Time;
            if (showmsg)
            {
                if (barTime <= suppTime)
                {
                    string msgID = "STV_VOL_status";
                    string msg = "STV_VOL_msg: " + (1 + (depth + distance) * jump) + " days of data have to be loaded";
                    DrawTextFixed(msgID, msg, TextPosition.BottomLeft,
                                Color.Blue, new Font("Arial", 6, FontStyle.Regular), Color.Empty, Color.Empty, 255);
                    return;
                }
                else
                {
                    string msgID = "STV_VOL_status";
                    string msg = "STV_VOL_msg: running";
                    DrawTextFixed(msgID, msg, TextPosition.BottomLeft,
                                Color.Blue, new Font("Arial", 6, FontStyle.Regular), Color.Empty, Color.Empty, 255);
                }
            }

            //
            double[] arrayVols = new double[depth];
            int mb = Bars.GetBar(barTime);      // 
            DateTime t = DateTime.Today;
            int barNbr = -1;
            int j = 0;
            for (int i = distance; i < distance + depth; ++i)
            {
                while (Bars.GetBar(barTime.AddDays(-(i * jump) - j)) == barNbr)  j++;    // jump w/e
                t = barTime.AddDays(-(i * jump) - j);    // i working days back
                barNbr = Bars.GetBar(t);        // get barNbr for i working day back

                int barIdx = mb - barNbr;       // calculate barIdx
                int movedIdx = barIdx - shift;      // moved bar idx
                double v = Volume[movedIdx];      // volume on barIdx = current bar - barNbr
                int valIdx = i - distance;      // array idx to set
                arrayVols[valIdx] = v;
             }

            double mean = 0;
            foreach (double average in arrayVols) mean += average;
            mean /= arrayVols.Length;
            double volAvg = mean;

            double volData = Volume[0];

            vd[0] = volData;
            va[0] = volAvg;

            Values[0].Set(va[0]);
            Values[1].Set(vd[0]);

        }

        private void RecalculateColors()
        {
            alpha = (int)(255.0 * ((100.0 - transparency) / 100.0));

            if (volumeAvgPen != null)
                volumeAvgPen = new Pen(Color.FromArgb(alpha, volumeAvgColor.R, volumeAvgColor.G, volumeAvgColor.B));
        }

		#region Properties
        [Browsable(false)]
        [XmlIgnore()]
        public DataSeries VA
        {
            get { return va; }
        }
        [Browsable(false)]	
        [XmlIgnore()]		
        public DataSeries VD
        {
            get { return vd; }
        }


        /// <summary>
        /// </summary>
        [Description("Show message")]
        [Category("Parameters")]
        public bool Showmsg
        {
            get { return showmsg; }
            set { showmsg = value; }
        }
        /// <summary>
        /// </summary>
        [Description("Shift bars")]
        [Category("Parameters")]
        public int Shift
        {
            get { return shift; }
            set { shift = value; }
        }
        /// <summary>
        /// </summary>
        [Description("Days multiplier")]
        [Category("Parameters")]
        public int Jump
        {
            get { return jump; }
            set { jump = Math.Max(1, value); }
        }
        /// <summary>
        /// </summary>
        [Description("Look days back for avg")]
        [Category("Parameters")]
        public int Depth
        {
            get { return depth; }
            set { depth = Math.Max(1, value); }
        }
        /// <summary>
        /// </summary>
        [Description("Exclude last from avg ")]
        [Category("Parameters")]
        public int Distance
        {
            get { return distance; }
            set { distance = Math.Max(1, value); }
        }

        /// <summary>
        /// </summary>
        [XmlIgnore()]
        [Description("Color of avg volume bars.")]
        [GridCategory("Colors")]
        [Gui.Design.DisplayNameAttribute("AvgVol color")]
        public Color VolumeAvgColor
        {
            get { return volumeAvgColor; }
            set { volumeAvgColor = value; RecalculateColors(); }
        }

        /// <summary>
        /// </summary>
        [Browsable(false)]
        public string VolumeAvgColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(volumeAvgColor); }
            set { volumeAvgColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        /// <summary>
        /// </summary>
        [XmlIgnore()]
        [Description("Color of the volume data line.")]
        [GridCategory("Colors")]
        [Gui.Design.DisplayNameAttribute("volume data color")]
        public Color VolumeDataColor
        {
            get { return volumeDataColor; }
            set { volumeDataColor = value; RecalculateColors(); }
        }

        /// <summary>
        /// </summary>
        [Browsable(false)]
        public string VolumeDataColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(volumeDataColor); }
            set { volumeDataColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }

        /// <summary>
        /// </summary>
        [Description("The percentage of transparency of the volume bars.")]
        [GridCategory("Parameters")]
        public int Transparency
        {
            get { return transparency; }
            set
            {
                transparency = Math.Max(0, value);
                transparency = Math.Min(90, value);
            }
        }

        #endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private STB_volavg[] cacheSTB_volavg = null;

        private static STB_volavg checkSTB_volavg = new STB_volavg();

        /// <summary>
        /// The STB_volavg (Volume Average) plots an average of volume.
        /// </summary>
        /// <returns></returns>
        public STB_volavg STB_volavg(int depth)
        {
            return STB_volavg(Input, depth);
        }

        /// <summary>
        /// The STB_volavg (Volume Average) plots an average of volume.
        /// </summary>
        /// <returns></returns>
        public STB_volavg STB_volavg(Data.IDataSeries input, int depth)
        {
            if (cacheSTB_volavg != null)
                for (int idx = 0; idx < cacheSTB_volavg.Length; idx++)
                    if (cacheSTB_volavg[idx].Depth == depth && cacheSTB_volavg[idx].EqualsInput(input))
                        return cacheSTB_volavg[idx];

            lock (checkSTB_volavg)
            {
                checkSTB_volavg.Depth = depth;
                depth = checkSTB_volavg.Depth;

                if (cacheSTB_volavg != null)
                    for (int idx = 0; idx < cacheSTB_volavg.Length; idx++)
                        if (cacheSTB_volavg[idx].Depth == depth && cacheSTB_volavg[idx].EqualsInput(input))
                            return cacheSTB_volavg[idx];

                STB_volavg indicator = new STB_volavg();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Depth = depth;
                Indicators.Add(indicator);
                indicator.SetUp();

                STB_volavg[] tmp = new STB_volavg[cacheSTB_volavg == null ? 1 : cacheSTB_volavg.Length + 1];
                if (cacheSTB_volavg != null)
                    cacheSTB_volavg.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSTB_volavg = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// The STB_volavg (Volume Average) plots an average of volume.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.STB_volavg STB_volavg(int depth)
        {
            return _indicator.STB_volavg(Input, depth);
        }

        /// <summary>
        /// TThe STB_volavg (Volume Average) plots an average of volume.
        /// </summary>
        /// <returns></returns>
        public Indicator.STB_volavg STB_volavg(Data.IDataSeries input, int depth)
        {
            return _indicator.STB_volavg(input, depth);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// The STB_volavg (Volume Average) plots an average of volume.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.STB_volavg STB_volavg(int depth)
        {
            return _indicator.STB_volavg(Input, depth);
        }

        /// <summary>
        /// The STB_volavg (Volume Moving Average) plots an exponential moving average (EMA) of volume.The STB_volavg (Volume Average) plots an average of volume.</summary>
        /// <returns></returns>
        public Indicator.STB_volavg STB_volavg(Data.IDataSeries input, int depth)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.STB_volavg(input, depth);
        }
    }
}
#endregion
